# Zope Navigator Product
# Copyright (C) 1999  Jonas Juselius
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# This product includes software developed by Digital Creations
# for use in the Z Object Publishing Environment
# (http://www.zope.org/).
#
_doc__="""Navigator 
$Id: Navigator.py,v 1.14 1999/10/18 13:51:01 jonas Exp $ """
__version__="1.02"


from Globals import Persistent, HTMLFile, MessageDialog, package_home
from OFS import SimpleItem 
from OFS.Folder import Folder
from OFS.ObjectManager import ObjectManager
from OFS.PropertyManager import PropertyManager
from AccessControl.Role import RoleManager
import Acquisition
from DateTime import DateTime
from OFS.Document import Document
import OFS.DTMLMethod 
import string, re, os, types

addNavigatorForm=HTMLFile('addNavigator', globals())
manage_navigator=HTMLFile('manageNavigator', globals())
addNavItemForm=HTMLFile('addNavItem', globals())
manage_navItemForm=HTMLFile('manageNavItem', globals())
miscPropertyForm=HTMLFile('miscPropertyForm', globals())

idx_html=re.compile('index_html$')

def manage_addNavigator(self, id, title='', ip='', render=None, REQUEST=None):
	"""Add a navigator object"""
	rid=id+'_render_dtml'
	top=1
	self._setObject(id, Navigator(id, rid, title, ip, top, REQUEST))
	if render:
		# Create the rendering engine
		prefix = package_home(globals())
		file = os.path.join(prefix, 'render.dtml')
		buf = open(file, 'r').read()
		i = Document(buf, __name__=rid)
		i.title='Change to determine how NavItems are rendered'
		try:
			rid=self._setObject(rid, i)
		except: pass
	if REQUEST: 
		return self.manage_main(self, REQUEST)

class Navigator(ObjectManager, PropertyManager, RoleManager, 
		SimpleItem.Item):
	"""Navigator class: Container of NavItems and Navigators"""

	meta_type = 'Navigator'
	id	  = 'Navigator'
	icon      = 'misc_/Navigator/navigator'
	meta_types=(
		{'name': 'NavItem', 'action' : 'manage_addNavItemForm'},
		{'name': 'Navigator', 'action' : 'manage_addNavigatorForm'},
		)

	# Specify definitions for tabs:
	manage_options=(
		{'label' : 'Contents', 'action' : 'manage_main'},
		{'label' : 'Misc', 'action' : 'manage_miscProperties'},
		{'label' : 'Properties', 'action' : 'manage_propertiesForm'},
		{'label' : 'Security', 'action' : 'manage_access'},)

	# Specify how individual operations add up to "permissions"
	__ac_permissions__=(
	    ('View management screens',
	    ['manage','manage_menu','manage_main','manage_copyright',
	    'manage_tabs','manage_propertiesForm','manage_navigator', 
	    'manage_misc']),
	)

	_properties=(
		{'id' : 'title', 'type': 'string', 'mode': 'w'},
		{'id' : 'icon_path', 'type': 'string', 'mode': 'w'},
		)

	def __init__(self, id, rid, title='', ip='', top=0, REQUEST=None):
		"Initialise a Navigator"
		self.id=id
		self.title=title
		self.icon_path=ip
		if REQUEST is None:
			raise ValueError, "REQUEST"
		if top == 1:
			self.postfix=''
			self.renderer=rid
			bp=string.strip(REQUEST['PATH_INFO'])
			idx=string.index(bp, '/manage_addNavigator')
			self.base_path=bp[:idx]
		if top == 2:
			self.renderer=rid

	manage=manage_main=manage_workspace=manage_navigator
	manage_miscProperties=miscPropertyForm

	def setprop(self, name, val):
		if hasattr(self.aq_self, name):
			if hasattr(self.aq_parent, name):
				if not val: 
					delattr(self, name)
				elif val == getattr(self.aq_parent, name):
					delattr(self, name)
			else:
				setattr(self, name, val)
		elif val:
			if val != getattr(self.aq_parent, name):
				setattr(self, name, val)
		
	def manage_misc(self, bp='', pf='', render='', REQUEST=None):
		"""Misc properties that should be acquired, unless explicitly
		set"""
		bp=string.strip(bp)
		pf=string.strip(pf)
		render=string.strip(render)
		self.setprop('base_path', bp)
		self.setprop('postfix', pf)
		self.setprop('renderer', render)
		if REQUEST:
			return MessageDialog(title='Modification Accepted',
				message='Your item has been added.',
				action = './manage_main' )
	
	def manage_addNavigator(self, id, title='', ip='', render=None, 
			REQUEST=None):
		"""Add a navigator object"""
		rid=id+'_render_dtml'
		if render: top=2
		else: top=0
		nav=Navigator(id, rid, title, ip, top, REQUEST)
		self._setObject(id, nav)
		if render:
			# Create the rendering engine
			prefix = package_home(globals())
			file = os.path.join(prefix, 'render.dtml')
			buf = open(file, 'r').read()
			i = Document(buf, __name__=rid)
			i.title='Change to determine how NavItems are rendered'
			try:
				rid=self._setObject(rid, i)
			except: pass
		if REQUEST: 
			return self.manage_main(self, REQUEST)

	def manage_addNavItem(self, id, title='', path='', ip='', 
		date='', expire='', new='0', level='1', pf='0', REQUEST=None):
		"Add a NavItem"
		obj=NavItem(id, title, path, ip, new, level, date, expire, pf)
		self._setObject(id, obj)
		if REQUEST:
			return MessageDialog(title='NavItem Accepted',
				message='Your item has been added.',
				action = './manage_main' )
	
	manage_addNavigatorForm=addNavigatorForm
	manage_addNavItemForm=addNavItemForm
	
	def __call__(self, section=None):
		"""Return self, for use with 'in, var and with' tags.
		   If called with a 'section' argument, return a Navigator
		   object, e.g. 'language' or whatever."""
		if section:
			ob=self._getOb(section)
			if ob.meta_type == 'Navigator':
				return ob
			raise AttributeError, 'Not a Navigator: '+ob.meta_type
		return self

	def has_attr(self, attr):
		"Return true if Navigator object has attribute 'attr'"
		if hasattr(self, attr):
			return 1
		return 0
	
	def navItems(self, exist=None): 
		"Like objectValues, but performs validification as well"
		l=[]
		for i in self.objectValues('NavItem'):
			if i.validate(exist):
				l.append(i)
		return l

	def ICON(self):
		"Return the full path to be used in the URL."
		if self.icon_path == '': return ''
		if self.icon_path[0] == '/':
			return self.icon_path
		return self.base_path+'/'+self.icon_path

	def Today(self):
		"return today"
		return DateTime().Date()
	
	
class NavItem(SimpleItem.Item, PropertyManager, RoleManager, Persistent,
	Acquisition.Implicit): 
	"""Navigator Item Class"""

	meta_type='NavItem'
	id='NavItem'
	icon='misc_/Navigator/navitem'
	
	manage_options=(
		{'label' : 'Main', 'action' : 'manage_main'},
		{'label' : 'Properties', 'action' : 'manage_propertiesForm'},
		{'label' : 'Security', 'action' : 'manage_access'},
		)

	_properties=(
		{'id' : 'title', 'type': 'string', 'mode': 'w'},
		{'id' : 'path', 'type': 'string', 'mode': 'w'},
		{'id' : 'icon_path', 'type': 'string', 'mode': 'w'},
		{'id' : 'non_active', 'type': 'boolean', 'mode': 'w'},
		)
	
	__ac_permissions__=(
	    ('View management screens',
	    ['manage','manage_menu','manage_main','manage_copyright',
	    'manage_tabs','manage_propertiesForm','manage_navItemForm']),
	)

	def __init__(self, id, title, path, ip, new, level, date, expire, pf):
		self.id=id

		if not string.strip(title):
			msg="Missing title!"
			raise ValueError, msg
		self.title=title
		self.path=string.strip(path)
		#if not path:
		#	raise ValueError, "Missing path!"
		#self.path=path
		
		if type(level) == types.StringType:
			level=string.atoi(level)
		if level < 1 or level > 5:
			raise ValueError, "Level out of range!"
		self.level=level
		
		if new == '1': self.new=1
		else: self.new=0

		if pf == '1': self.pf=1
		else: self.pf=0

		if string.strip(expire): self.expire=DateTime(expire)
		else: self.expire=''
		
		if date: self.date=DateTime(date)
		else: self.date=DateTime()
		
		if ip: self.icon_path=ip
		else: self.icon_path=''

		self.non_active=0
	
	def PATH(self, strip=0):
		"Return the full path to be used in the URL."
		if not self.path: 
			if strip:
				return self.base_path
			elif self.pf:
				return self.base_path+self.postfix
			else:
				return self.base_path
		idx=string.find(self.path, '?')
		if strip:
			if idx == -1: path=self.path
			else: path=self.path[:idx]
		elif self.pf:
			fix=string.strip(self.postfix)
			if fix:
				if idx == -1: path=self.path+'?'+fix
				else: path=self.path+'&'+fix
			else:
				path=self.path
		else:
			path=self.path
		
		if path[0] != '/':
			path=self.base_path+'/'+path
		return path
	
	def ICON(self):
		"Return the full path to be used in the URL."
		if self.icon_path == '': return ''
		if self.icon_path[0] == '/':
			return self.icon_path
		return self.base_path+'/'+self.icon_path

	manage=manage_main=manage_workspace=manage_navItemForm

	def renderItem(self, REQUEST=None):
		"""Render NavItem"""
		if not hasattr(self, self.renderer):
			return "<a href=%s> %s</a>" % (self.PATH(),
				self.title)
		renderer=getattr(self, self.renderer)
		return renderer(PATH=self.PATH(),
			title    = self.title,
			level    = self.level,
			date     = self.date, 
			expire   = self.expire,
			new      = self.new,
			ICON     = self.ICON(), 
			id       = self.id,
			inPath   = self.inPath(REQUEST), 
			isActive = self.isActive(REQUEST),
			REQUEST=REQUEST)
	
	def manage_editNavItem(self, title='', path='', ip='', date='', 
		expire='', level='1', new='0', pf='0', REQUEST=None):
		"Edit item"
		
		if not string.strip(title):
			msg="Missing title!"
			raise ValueError, msg
		self.title=title
		path=string.strip(path)
		#if not path:
		#	raise ValueError, "Missing path!"
		self.path=path
		
		if type(level) == types.StringType:
			level=string.atoi(level)
		if level < 1 or level > 5:
			raise ValueError, "Level out of range!"
		self.level=level
		
		if new == '1': self.new=1
		else: self.new=0

		if pf == '1': self.pf=1
		else: self.pf=0

		if string.strip(expire): self.expire=DateTime(expire)
		else: self.expire=''
		
		if date: self.date=DateTime(date)
		else: self.date=DateTime()
		
		if ip: self.icon_path=ip
		else: self.icon_path=''

		#if not self.exists(self.PATH(strip=1)):
		#	raise ValueError, "Invalid Path"

		#if not self.exists(self.ICON()):
		#	raise ValueError, "Invalid Icon Path!"

		if REQUEST:
			return MessageDialog(title='Edited',
				message='Entry has been edited.',
				action ='./manage_main',)
		
	def strip_index_html(self, path):
		return idx_html.sub('', path)
		
	def isActive(self, REQUEST):
		"""Returns true if PATH() equals to PATH_INFO""" 
		if self.non_active: return 0
		p_info=string.strip(REQUEST['PATH_INFO'])
		p=self.PATH(strip=1)
		p_info=self.strip_index_html(p_info)
		p=self.strip_index_html(p)
		if p_info[-1] == '/':
			p_info=p_info[:-1]
		if p[-1] == '/':
			p=p[:-1]
		if p == p_info:
			return 1
		return 0
	
	def inPath(self, REQUEST):
		"""Returns true if PATH() is in PATH_INFO""" 
		# Top level is a problem, always returns true...
		if self.isActive(REQUEST):
			return 1
		l=len(REQUEST['SERVER_URL'])
		p_info=REQUEST['URL0']
		p_info=p_info[l:]
		if REQUEST.has_key('toplevel'):
			top=REQUEST['toplevel']
		else:
			top='/'
		if self.path == top:
			i=string.rfind(p_info, '/')
			if p_info[:i+1] == top: 
				return 1
			return 0
		if self.path and self.path[0] == '/':
			path=re.escape(self.path)
		else:
			path=re.escape(self.base_path)
		path=re.compile(path)
		if path.match(REQUEST['PATH_INFO']):
			return 1
		return 0
	
	def getObject(self):
		"Return the actual object PATH is refering to."
		return self.resolve_url(self.PATH(), self.REQUEST)
	
	def exists(self, path): 
		try:
			self.resolve_url(path, self.REQUEST)
		except:
			return 0
		return 1

	def validate(self, exist=None): 
		"Check if the link is publishable"
		if exist:
			if not self.exists(self.PATH(strip=1)):
				return 0
		if self.expire:
			if DateTime().greaterThan(self.expire):
				return 0
		if DateTime().lessThan(self.date):
			return 0
		return 1

	__call__=renderItem

## stolen from ZPublisher and modified slightly
    
	def resolve_url(self, path, REQUEST):
        # Attempt to resolve a url into an object in the Zope
        # namespace. The url must be a fully-qualified url. The
        # method will return the requested object if it is found
        # or raise the same HTTP error that would be raised in
        # the case of a real web request. If the passed in url
        # does not appear to describe an object in the system
        # namespace (e.g. the host, port or script name dont
        # match that of the current request), a ValueError will
        # be raised.

		while path and path[0]=='/':  path=path[1:]
		while path and path[-1]=='/': path=path[:-1]
		req=REQUEST.clone()
		rsp=req.response
		req['PATH_INFO']=path
		object=None
		try: 
			object=req.traverse(path)
		except: 
			rsp.exception(0)
		if object is None:
			req.close()
			raise rsp.errmsg, sys.exc_value

		if hasattr(object, 'id'):
			if callable(object.id):
				name=object.id()
			else: 
				name=object.id
		elif hasattr(object, '__name__'):
			name=object.__name__
		else: name=''
	
		if name != os.path.split(path)[-1]:
			result = req.PARENTS[0]
			req.close()
			return result
		req.close()
		return object

	def Today(self):
		"return today"
		return DateTime().Date()

